@@ -49,7 +49,9 @@
      STRING = 265,
      CONFIG = 266,
      VALUE = 267,
-     NUMBER = 268
+     NUMBER = 268,
+     SETUP = 269,
+     FID = 270
    };
 #endif
 /* Tokens.  */
@@ -64,13 +66,15 @@
 #define CONFIG 266
 #define VALUE 267
 #define NUMBER 268
+#define SETUP 269
+#define FID 270
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 63 "./parser.y"
+#line 130 "parser.y"
 {
 	re2c::Symbol	*symbol;
 	re2c::RegExp	*regexp;
@@ -82,7 +86,7 @@ typedef union YYSTYPE
 	re2c::CondList	*clist;
 }
 /* Line 1489 of yacc.c.  */
-#line 86 "y.tab.h"
+#line 90 "y.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
