@@ -1,4 +1,4 @@
-/* $Id: code.cc 750 2007-05-06 22:55:26Z helly $ */
+/* $Id: code.cc 834 2008-03-13 17:38:12Z helly $ */
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -22,7 +22,7 @@ static std::string indent(uint ind)
 {
 	std::string str;
 
-	while (ind-- > 0)
+	while (!DFlag && ind-- > 0)
 	{
 		str += indString;
 	}
@@ -46,7 +46,7 @@ std::string replaceParam(std::string str, const std::string& param, const _Ty& v
 	return str;
 }
 
-static void genYYFill(std::ostream &o, uint ind, uint need)
+static void genYYFill(std::ostream &o, uint, uint need)
 {
 	if (bUseYYFillParam)
 	{
@@ -288,7 +288,7 @@ const BitMap *BitMap::find(const State *x)
 
 void BitMap::gen(std::ostream &o, uint ind, uint lb, uint ub)
 {
-	if (first && bLastPass)
+	if (first && bLastPass && bUsedYYBitmap)
 	{
 		o << indent(ind) << "static const unsigned char " << mapCodeName["yybm"] << "[] = {";
 
@@ -362,6 +362,12 @@ void BitMap::stats()
 
 void genGoTo(std::ostream &o, uint ind, const State *from, const State *to, bool & readCh)
 {
+	if (DFlag)
+	{
+		o << from->label << " -> " << to->label << "\n";
+		return;
+	}
+
 	if (readCh && from->label + 1 != to->label)
 	{
 		o << indent(ind) << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ";\n";
@@ -374,7 +380,7 @@ void genGoTo(std::ostream &o, uint ind, const State *from, const State *to, bool
 
 void genIf(std::ostream &o, uint ind, const char *cmp, uint v, bool &readCh)
 {
-	o << indent(ind) << "if(";
+	o << indent(ind) << "if (";
 	if (readCh)
 	{
 		o << "(" << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ")";
@@ -392,6 +398,11 @@ void genIf(std::ostream &o, uint ind, const char *cmp, uint v, bool &readCh)
 
 static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMarker)
 {
+	if (DFlag)
+	{
+		return;
+	}
+
 	uint fillIndex = next_fill_index;
 
 	if (fFlag)
@@ -411,12 +422,12 @@ static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMark
 	{
 		if (n == 1)
 		{
-			o << indent(ind) << "if(" << mapCodeName["YYLIMIT"] << " == " << mapCodeName["YYCURSOR"] << ") ";
+			o << indent(ind) << "if (" << mapCodeName["YYLIMIT"] << " == " << mapCodeName["YYCURSOR"] << ") ";
 			genYYFill(o, ind, n);
 		}
 		else
 		{
-			o << indent(ind) << "if((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") ";
+			o << indent(ind) << "if ((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") ";
 			genYYFill(o, ind, n);
 		}
 	}
@@ -440,8 +451,13 @@ static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMark
 	}
 }
 
-void Match::emit(std::ostream &o, uint ind, bool &readCh) const
+void Match::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) const
 {
+	if (DFlag)
+	{
+		return;
+	}
+
 	if (state->link)
 	{
 		o << indent(ind) << "++" << mapCodeName["YYCURSOR"] << ";\n";
@@ -464,7 +480,7 @@ void Match::emit(std::ostream &o, uint ind, bool &readCh) const
 	}
 }
 
-void Enter::emit(std::ostream &o, uint ind, bool &readCh) const
+void Enter::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) const
 {
 	if (state->link)
 	{
@@ -487,7 +503,7 @@ void Enter::emit(std::ostream &o, uint ind, bool &readCh) const
 	}
 }
 
-void Initial::emit(std::ostream &o, uint ind, bool &readCh) const
+void Initial::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) const
 {
 	if (!cFlag && !startLabelName.empty())
 	{
@@ -534,8 +550,13 @@ void Initial::emit(std::ostream &o, uint ind, bool &readCh) const
 	}
 }
 
-void Save::emit(std::ostream &o, uint ind, bool &readCh) const
+void Save::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) const
 {
+	if (DFlag)
+	{
+		return;
+	}
+
 	if (bUsedYYAccept)
 	{
 		o << indent(ind) << mapCodeName["yyaccept"] << " = " << selector << ";\n";
@@ -568,7 +589,7 @@ Move::Move(State *s) : Action(s)
 	;
 }
 
-void Move::emit(std::ostream &, uint, bool &) const
+void Move::emit(std::ostream &, uint, bool &, const std::string&) const
 {
 	;
 }
@@ -596,7 +617,8 @@ void Accept::emitBinary(std::ostream &o, uint ind, uint l, uint r, bool &readCh)
 	{
 		uint m = (l + r) >> 1;
 
-		o << indent(ind) << "if(" << mapCodeName["yyaccept"] << " <= " << m << ") {\n";
+		assert(bUsedYYAccept);
+		o << indent(ind) << "if (" << mapCodeName["yyaccept"] << " <= " << m << ") {\n";
 		emitBinary(o, ++ind, l, m, readCh);
 		o << indent(--ind) << "} else {\n";
 		emitBinary(o, ++ind, m + 1, r, readCh);
@@ -608,12 +630,15 @@ void Accept::emitBinary(std::ostream &o, uint ind, uint l, uint r, bool &readCh)
 	}
 }
 
-void Accept::emit(std::ostream &o, uint ind, bool &readCh) const
+void Accept::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) const
 {
 	if (mapRules.size() > 0)
 	{
 		bUsedYYMarker = true;
-		o << indent(ind) << mapCodeName["YYCURSOR"] << " = " << mapCodeName["YYMARKER"] << ";\n";
+		if (!DFlag)
+		{
+			o << indent(ind) << mapCodeName["YYCURSOR"] << " = " << mapCodeName["YYMARKER"] << ";\n";
+		}
 
 		if (readCh) // shouldn't be necessary, but might become at some point
 		{
@@ -642,9 +667,17 @@ void Accept::emit(std::ostream &o, uint ind, bool &readCh) const
 			{
 				emitBinary(o, ind, 0, mapRules.size() - 1, readCh);
 			}
+			else if (DFlag)
+			{
+				for (RuleMap::const_iterator it = mapRules.begin(); it != mapRules.end(); ++it)
+				{
+					o << state->label << " -> " << it->second->label;
+					o << " [label=\"yyaccept=" << it->first << "\"]\n";
+				}
+			}
 			else
 			{
-				o << indent(ind) << "switch(" << mapCodeName["yyaccept"] << ") {\n";
+				o << indent(ind) << "switch (" << mapCodeName["yyaccept"] << ") {\n";
 		
 				for (RuleMap::const_iterator it = mapRules.begin(); it != mapRules.end(); ++it)
 				{
@@ -668,8 +701,14 @@ Rule::Rule(State *s, RuleOp *r) : Action(s), rule(r)
 	;
 }
 
-void Rule::emit(std::ostream &o, uint ind, bool &) const
+void Rule::emit(std::ostream &o, uint ind, bool &, const std::string& condName) const
 {
+	if (DFlag)
+	{
+		o << state->label << " [label=\"" << sourceFileInfo.fname << ":" << rule->code->line << "\"];\n";
+		return;
+	}
+
 	uint back = rule->ctx->fixedLength();
 
 	if (back != 0u)
@@ -677,16 +716,28 @@ void Rule::emit(std::ostream &o, uint ind, bool &) const
 		o << indent(ind) << mapCodeName["YYCURSOR"] << " = " << mapCodeName["YYCTXMARKER"] << ";\n";
 	}
 
-	if (rule->code->newcond)
+	if (rule->code->newcond.length() && condName != rule->code->newcond)
 	{
-		genSetCondition(o, ind, rule->code->newcond->to_string());
+		genSetCondition(o, ind, rule->code->newcond);
 	}
 
 	RuleLine rl(*rule);
 
+	if (!yySetupRule.empty() && !rule->code->autogen)
+	{
+		o << indent(ind) << yySetupRule << "\n";
+	}
+
 	o << file_info(sourceFileInfo, &rl);
 	o << indent(ind);
-	o << rule->code->text;
+	if (rule->code->autogen)
+	{
+		o << replaceParam(condGoto, condGotoParam, condPrefix + rule->code->newcond);
+	}
+	else
+	{
+		o << rule->code->text;
+	}
 	o << "\n";
 	o << outputFileInfo;
 }
@@ -706,7 +757,7 @@ void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, con
 					genIf(o, ind, "!=", s[0].ub, readCh);
 					genGoTo(o, 0, from, bg, readCh);
 				}
-				if (next->label != from->label + 1)
+				if (next->label != from->label + 1 || DFlag)
 				{
 					genGoTo(o, ind, from, next, readCh);
 				}
@@ -728,7 +779,7 @@ void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, con
 		if (n == 1)
 		{
 			//	    	if(bg != next){
-			if (s[0].to->label != from->label + 1)
+			if (s[0].to->label != from->label + 1 || DFlag)
 			{
 				genGoTo(o, ind, from, s[0].to, readCh);
 			}
@@ -742,7 +793,7 @@ void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, con
 				genIf(o, ind, ">=", s[0].ub, readCh);
 				genGoTo(o, 0, from, s[1].to, readCh);
 			}
-			if (next->label != from->label + 1)
+			if (next->label != from->label + 1 || DFlag)
 			{
 				genGoTo(o, ind, from, next, readCh);
 			}
@@ -760,7 +811,7 @@ void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, con
 		}
 	}
 
-	if (next->label != from->label + 1)
+	if (next->label != from->label + 1 || DFlag)
 	{
 		genGoTo(o, ind, from, next, readCh);
 	}
@@ -771,7 +822,7 @@ void Go::genLinear(std::ostream &o, uint ind, const State *from, const State *ne
 	doLinear(o, ind, span, nSpans, from, next, readCh, mask);
 }
 
-bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint mask)
+bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint mask, const State *from, const State *to)
 {
 	bool used = false;
 
@@ -786,9 +837,19 @@ bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint m
 		{
 			if (!mask || lb > 0x00FF)
 			{
-				o << indent(ind) << "case ";
-				prtChOrHex(o, lb);
-				o << ":";
+				if (DFlag)
+				{
+					o << from->label << " -> " << to->label;
+					o << " [label=";
+					prtChOrHex(o, lb);
+					o << "]";
+				}
+				else
+				{
+					o << indent(ind) << "case ";
+					prtChOrHex(o, lb);
+					o << ":";
+				}
 				newLine = false;
 				used = true;
 			}
@@ -828,19 +889,22 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 			}
 		}
 
-		if (dFlag)
+		if (!DFlag)
 		{
-			o << indent(ind) << mapCodeName["YYDEBUG"] << "(-1, " << mapCodeName["yych"] << ");\n";
-		}
+			if (dFlag)
+			{
+				o << indent(ind) << mapCodeName["YYDEBUG"] << "(-1, " << mapCodeName["yych"] << ");\n";
+			}
 
-		if (readCh)
-		{
-			o << indent(ind) << "switch((" << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ")) {\n";
-			readCh = false;
-		}
-		else
-		{
-			o << indent(ind) << "switch(" << mapCodeName["yych"] << ") {\n";
+			if (readCh)
+			{
+				o << indent(ind) << "switch ((" << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ")) {\n";
+				readCh = false;
+			}
+			else
+			{
+				o << indent(ind) << "switch (" << mapCodeName["yych"] << ") {\n";
+			}
 		}
 
 		while (t != &sP[0])
@@ -849,22 +913,22 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 
 			r = s = &sP[0];
 
+			const State *to = (*s)->to;
+
 			if (*s == &span[0])
 			{
-				used |= genCases(o, ind, 0, *s, newLine, mask);
+				used |= genCases(o, ind, 0, *s, newLine, mask, from, to);
 			}
 			else
 			{
-				used |= genCases(o, ind, (*s)[ -1].ub, *s, newLine, mask);
+				used |= genCases(o, ind, (*s)[ -1].ub, *s, newLine, mask, from, to);
 			}
 
-			State *to = (*s)->to;
-
 			while (++s < t)
 			{
 				if ((*s)->to == to)
 				{
-					used |= genCases(o, ind, (*s)[ -1].ub, *s, newLine, mask);
+					used |= genCases(o, ind, (*s)[ -1].ub, *s, newLine, mask, from, to);
 				}
 				else
 				{
@@ -872,7 +936,7 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 				}
 			}
 
-			if (used)
+			if (used && !DFlag)
 			{
 				genGoTo(o, newLine ? ind+1 : 1, from, to, readCh);
 				newLine = true;
@@ -880,9 +944,17 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 			t = r;
 		}
 
-		o << indent(ind) << "default:";
-		genGoTo(o, 1, from, def, readCh);
-		o << indent(ind) << "}\n";
+		if (DFlag)
+		{
+			o << "\n" << from->label << " -> " << def->label;
+			o << " [label=default]\n" ;
+		}
+		else
+		{
+			o << indent(ind) << "default:";
+			genGoTo(o, 1, from, def, readCh);
+			o << indent(ind) << "}\n";
+		}
 
 		delete [] sP;
 	}
@@ -998,7 +1070,7 @@ void Go::genCpGoto(std::ostream &o, uint ind, const State *from, const State *ne
 	readCh = false;
 	if (wFlag)
 	{
-		o << indent(ind) << "if(" << sYych <<" & ~0xFF) {\n";
+		o << indent(ind) << "if (" << sYych <<" & ~0xFF) {\n";
 		genBase(o, ind+1, from, next, readCh, 1);
 		o << indent(ind++) << "} else {\n";
 		sYych = mapCodeName["yych"];
@@ -1115,7 +1187,7 @@ void Go::genGoto(std::ostream &o, uint ind, const State *from, const State *next
 					readCh = false;
 					if (wFlag)
 					{
-						o << indent(ind) << "if(" << sYych << " & ~0xFF) {\n";
+						o << indent(ind) << "if (" << sYych << " & ~0xFF) {\n";
 						sYych = mapCodeName["yych"];
 						genBase(o, ind+1, from, next, readCh, 1);
 						o << indent(ind) << "} else ";
@@ -1124,7 +1196,8 @@ void Go::genGoto(std::ostream &o, uint ind, const State *from, const State *next
 					{
 						o << indent(ind);
 					}
-					o << "if(" << mapCodeName["yybm"] << "[" << b->i << "+" << sYych << "] & ";
+					bUsedYYBitmap = true;
+					o << "if (" << mapCodeName["yybm"] << "[" << b->i << "+" << sYych << "] & ";
 					if (yybmHexTable)
 					{
 						prtHex(o, b->m, false);
@@ -1147,7 +1220,7 @@ void Go::genGoto(std::ostream &o, uint ind, const State *from, const State *next
 	genBase(o, ind, from, next, readCh, 0);
 }
 
-void State::emit(std::ostream &o, uint ind, bool &readCh) const
+void State::emit(std::ostream &o, uint ind, bool &readCh, const std::string& condName) const
 {
 	if (vUsedLabels.count(label))
 	{
@@ -1161,7 +1234,7 @@ void State::emit(std::ostream &o, uint ind, bool &readCh) const
 	{
 		o << indent(ind) << mapCodeName["YYCTXMARKER"] << " = " << mapCodeName["YYCURSOR"] << " + 1;\n";
 	}
-	action->emit(o, ind, readCh);
+	action->emit(o, ind, readCh, condName);
 }
 
 uint merge(Span *x0, State *fg, State *bg)
@@ -1239,7 +1312,7 @@ uint merge(Span *x0, State *fg, State *bg)
 	}
 }
 
-const uint cInfinity = ~0;
+const uint cInfinity = ~0u;
 
 class SCC
 {
@@ -1480,6 +1553,8 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	State *s;
 	uint i;
 
+	bUsedYYBitmap = false;
+
 	findSCCs();
 	head->link = head;
 
@@ -1611,7 +1686,10 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	delete head->action;
 	head->action = NULL;
 
-	bUsedYYAccept = false;
+	if (!cFlag)
+	{
+		bUsedYYAccept = false;
+	}
 	
 	uint start_label = next_label;
 
@@ -1638,7 +1716,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	for (s = head; s; s = s->next)
 	{
 		bool readCh = false;
-		s->emit(null_dev, ind, readCh);
+		s->emit(null_dev, ind, readCh, condName);
 		s->go.genGoto(null_dev, ind, s, s->next, readCh);
 	}
 	if (last_fill_index < next_fill_index)
@@ -1651,7 +1729,13 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	if (bProlog)
 	{
 		o << "\n" << outputFileInfo;
-		if ((!fFlag && bUsedYYAccept)
+
+		if (DFlag)
+		{
+			bPrologBrace = true;
+			o << "digraph re2c {\n";
+		}
+		else if ((!fFlag && bUsedYYAccept)
 		||  (!fFlag && bEmitYYCh)
 		||  (bFlag && !cFlag && BitMap::first)
 		||  (cFlag && !bWroteCondCheck && gFlag && !specMap->empty())
@@ -1665,8 +1749,8 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 		{
 			ind = 1;
 		}
-	
-		if (!fFlag)
+
+		if (!fFlag && !DFlag)
 		{
 			if (bEmitYYCh)
 			{
@@ -1674,7 +1758,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 			}
 			if (bUsedYYAccept)
 			{
-				o << indent(ind) << "unsigned int "<< mapCodeName["yyaccept"] << " = 0;\n";
+				o << indent(ind) << "unsigned int " << mapCodeName["yyaccept"] << " = 0;\n";
 			}
 		}
 		else
@@ -1682,7 +1766,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 			o << "\n";
 		}
 	}
-	if (bFlag && !cFlag)
+	if (bFlag && !cFlag && BitMap::first)
 	{
 		BitMap::gen(o, ind, lbChar, ubChar <= 256 ? ubChar : 256);
 	}
@@ -1706,9 +1790,19 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 
 	if (cFlag && !condName.empty())
 	{
-		// TODO: Drop marker
-		o << "/* *********************************** */\n";
-		o << condPrefix << condName << ":\n";
+		if (condDivider.length())
+		{
+			o << replaceParam(condDivider, condDividerParam, condName) << "\n";
+		}
+
+		if (DFlag)
+		{
+			o << condName << " -> " << (start_label+1) << "\n";
+		}
+		else
+		{
+			o << condPrefix << condName << ":\n";
+		}
 	}
 	if (cFlag && bFlag && BitMap::first)
 	{
@@ -1727,7 +1821,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	for (s = head; s; s = s->next)
 	{
 		bool readCh = false;
-		s->emit(o, ind, readCh);
+		s->emit(o, ind, readCh, condName);
 		s->go.genGoto(o, ind, s, s->next, readCh);
 	}
 
@@ -1796,23 +1890,22 @@ void genGetStateGoto(std::ostream &o, uint& ind, uint start_label)
 			o << indent(--ind) << "};\n";
 			o << "\n";
 
-			o << indent(ind) << "if(" << genGetState();
+			o << indent(ind) << "if (" << genGetState();
 			if (bUseStateAbort)
 			{
-				o << " == -1) {";
-				++ind;
+				o << " == -1) {\n";
 			}
 			else
 			{
-				o << " < 0)";
+				o << " < 0) {\n";
 			}
-			o << " goto " << labelPrefix << start_label << ";\n";
+			o << indent(++ind) << "goto " << labelPrefix << start_label << ";\n";
 			if (bUseStateAbort)
 			{
 				o << indent(--ind) << "} else if (" << genGetState() << " < -1) {\n";
 				o << indent(++ind) << "abort();\n";
-				o << indent(--ind) << "}\n";
 			}
+			o << indent(--ind) << "}\n";
 
 			o << indent(ind) << "goto *" << mapCodeName["yystable"] << "[" << genGetState() << "];\n";
 		}
@@ -1826,7 +1919,7 @@ void genGetStateGoto(std::ostream &o, uint& ind, uint start_label)
 		}
 		else
 		{
-			o << indent(ind) << "switch(" << genGetState() << ") {\n";
+			o << indent(ind) << "switch (" << genGetState() << ") {\n";
 			if (bUseStateAbort)
 			{
 				o << indent(ind) << "default: abort();\n";
@@ -1911,9 +2004,16 @@ void genCondGoto(std::ostream &o, uint ind, const RegExpMap& specMap)
 				}
 				genCondGotoSub(o, ind, vCondList, 0, vCondList.size() - 1);
 			}
+			else if (DFlag)
+			{
+				for(RegExpMap::const_iterator it = specMap.begin(); it != specMap.end(); ++it)
+				{
+					o << "0 -> " << it->first << " [label=\"state=" << it->first << "\"]\n";
+				}
+			}
 			else
 			{
-				o << indent(ind) << "switch(" << genGetCondition() << ") {\n";
+				o << indent(ind) << "switch (" << genGetCondition() << ") {\n";
 	
 				for(RegExpMap::const_iterator it = specMap.begin(); it != specMap.end(); ++it)
 				{
@@ -1936,6 +2036,8 @@ void genTypes(std::string& o, uint ind, const RegExpMap& specMap)
 
 	for(RegExpMap::const_iterator itSpecMap = specMap.begin(); itSpecMap != specMap.end(); ++itSpecMap)
 	{
+		// If an entry is < 0 then we did the 0/empty correction twice.
+		assert(itSpecMap->second.first >= 0);
 		vCondList[itSpecMap->second.first] = itSpecMap->first;
 	}
 
@@ -2119,6 +2221,22 @@ void Scanner::config(const Str& cfg, const Str& val)
 	{
 		condEnumPrefix = strVal;
 	}
+	else if (cfg.to_string() == "cond:divider")
+	{
+		condDivider = strVal;
+	}
+	else if (cfg.to_string() == "cond:divider@cond")
+	{
+		condDividerParam = strVal;
+	}
+	else if (cfg.to_string() == "cond:goto")
+	{
+		condGoto = strVal;
+	}
+	else if (cfg.to_string() == "cond:goto@cond")
+	{
+		condGotoParam = strVal;
+	}
 	else if (cfg.to_string() == "define:YYFILL@len")
 	{
 		yyFillLength = strVal;
@@ -2136,7 +2254,7 @@ void Scanner::config(const Str& cfg, const Str& val)
 	}
 	else if (mapVariableKeys.find(cfg.to_string()) != mapVariableKeys.end())
     {
-    	if (bFirstPass && !mapCodeName.insert(
+    	if ((bFirstPass || rFlag) && !mapCodeName.insert(
     			std::make_pair(cfg.to_string().substr(sizeof("variable:") - 1), strVal)
     			).second)
     	{
@@ -2145,7 +2263,7 @@ void Scanner::config(const Str& cfg, const Str& val)
     }
 	else if (mapDefineKeys.find(cfg.to_string()) != mapDefineKeys.end())
     {
-    	if (bFirstPass && !mapCodeName.insert(
+    	if ((bFirstPass || rFlag) && !mapCodeName.insert(
     			std::make_pair(cfg.to_string().substr(sizeof("define:") - 1), strVal)
     			).second)
     	{
@@ -2154,7 +2272,7 @@ void Scanner::config(const Str& cfg, const Str& val)
     }
 	else if (mapLabelKeys.find(cfg.to_string()) != mapLabelKeys.end())
     {
-    	if (bFirstPass && !mapCodeName.insert(
+    	if ((bFirstPass || rFlag) && !mapCodeName.insert(
     			std::make_pair(cfg.to_string().substr(sizeof("label:") - 1), strVal)
     			).second)
     	{
