@@ -1,4 +1,4 @@
-/* $Id: globals.h 727 2007-05-01 14:55:50Z helly $ */
+/* $Id: globals.h 829 2008-03-07 21:59:19Z helly $ */
 #ifndef	_globals_h
 #define	_globals_h
 
@@ -19,10 +19,13 @@ extern file_info headerFileInfo;
 extern bool bFlag;
 extern bool cFlag;
 extern bool dFlag;
+extern bool DFlag;
 extern bool eFlag;
 extern bool fFlag;
+extern bool FFlag;
 extern bool gFlag;
 extern bool iFlag;
+extern bool rFlag;
 extern bool sFlag;
 extern bool tFlag;
 extern bool uFlag;
@@ -37,16 +40,22 @@ extern bool bLastPass;
 extern bool bUsedYYAccept;
 extern bool bUsedYYMaxFill;
 extern bool bUsedYYMarker;
+extern bool bUsedYYBitmap;
 
 extern bool bUseStartLabel;
 extern std::string startLabelName;
 extern std::string labelPrefix;
 extern std::string condPrefix;
 extern std::string condEnumPrefix;
+extern std::string condDivider;
+extern std::string condDividerParam;
+extern std::string condGoto;
+extern std::string condGotoParam;
 extern std::string yychConversion;
 extern std::string yyFillLength;
 extern std::string yySetConditionParam;
 extern std::string yySetStateParam;
+extern std::string yySetupRule;
 extern uint maxFill;
 extern uint next_label;
 extern uint cGotoThreshold;
@@ -67,6 +76,9 @@ extern bool bUseYYSetStateParam;
 extern bool bUseYYGetStateNaked;
 extern bool bWroteGetState;
 extern bool bWroteCondCheck;
+extern bool bCaseInsensitive;
+extern bool bCaseInverted;
+extern bool bTypesDone;
 
 extern uint asc2ebc[256];
 extern uint ebc2asc[256];
