@@ -1,4 +1,4 @@
-/* $Id: main.cc 727 2007-05-01 14:55:50Z helly $ */
+/* $Id: main.cc 829 2008-03-07 21:59:19Z helly $ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #elif defined(_WIN32)
@@ -25,10 +25,13 @@ file_info headerFileInfo;
 bool bFlag = false;
 bool cFlag = false;
 bool dFlag = false;
+bool DFlag = false;
 bool eFlag = false;
 bool fFlag = false;
+bool FFlag = false;
 bool gFlag = false;
 bool iFlag = false;
+bool rFlag = false;
 bool sFlag = false;
 bool tFlag = false;
 bool uFlag = false;
@@ -39,6 +42,7 @@ bool bNoGenerationDate = false;
 bool bSinglePass = false;
 bool bFirstPass  = true;
 bool bLastPass   = false;
+bool bUsedYYBitmap  = false;
 
 bool bUsedYYAccept  = false;
 bool bUsedYYMaxFill = false;
@@ -59,10 +63,15 @@ std::string startLabelName;
 std::string labelPrefix("yy");
 std::string condPrefix("yyc_");
 std::string condEnumPrefix("yyc");
+std::string condDivider("/* *********************************** */");
+std::string condDividerParam("@@");
+std::string condGoto("goto @@;");
+std::string condGotoParam("@@");
 std::string yychConversion("");
 std::string yyFillLength("@@");
 std::string yySetConditionParam("@@");
 std::string yySetStateParam("@@");
+std::string yySetupRule("");
 uint maxFill = 1;
 uint next_label = 0;
 uint cGotoThreshold = 9;
@@ -73,6 +82,9 @@ bool yybmHexTable = false;
 bool bUseStateAbort = false;
 bool bWroteGetState = false;
 bool bWroteCondCheck = false;
+bool bCaseInsensitive = false;
+bool bCaseInverted = false;
+bool bTypesDone = false;
 
 uint nRealChars = 256;
 
@@ -96,26 +108,31 @@ static const mbo_opt_struct OPTIONS[] =
 	mbo_opt_struct('b', 0, "bit-vectors"),
 	mbo_opt_struct('c', 0, "start-conditions"),
 	mbo_opt_struct('d', 0, "debug-output"),
+	mbo_opt_struct('D', 0, "emit-dot"),
 	mbo_opt_struct('e', 0, "ecb"),
 	mbo_opt_struct('f', 0, "storable-state"),
+	mbo_opt_struct('F', 0, "flex-syntax"),
 	mbo_opt_struct('g', 0, "computed-gotos"),
 	mbo_opt_struct('h', 0, "help"),
 	mbo_opt_struct('i', 0, "no-debug-info"),
 	mbo_opt_struct('o', 1, "output"),
+	mbo_opt_struct('r', 0, "reusable"),
 	mbo_opt_struct('s', 0, "nested-ifs"),
 	mbo_opt_struct('t', 1, "type-header"),
 	mbo_opt_struct('u', 0, "unicode"),
 	mbo_opt_struct('v', 0, "version"),
 	mbo_opt_struct('V', 0, "vernum"),
-	mbo_opt_struct('w', 0, "wide-chars"),      
+	mbo_opt_struct('w', 0, "wide-chars"),
 	mbo_opt_struct('1', 0, "single-pass"),
 	mbo_opt_struct(10,  0, "no-generation-date"),
+	mbo_opt_struct(11,  0, "case-insensitive"),
+	mbo_opt_struct(12,  0, "case-inverted"),
 	mbo_opt_struct('-', 0, NULL) /* end of args */
 };
 
 static void usage()
 {
-	cerr << "usage: re2c [-bdefghisvVw1] [-o file] file\n"
+	cerr << "usage: re2c [-bcdDefFghisvVw1] [-o file] file\n"
 	"\n"
 	"-? -h  --help           Display this info.\n"
 	"\n"
@@ -130,11 +147,15 @@ static void usage()
 	"                        about the current position and in which state the\n"
 	"                        parser is.\n"
 	"\n"
+	"-D     --emit-dot       Emit a Graphviz dot view of the DFA graph\n"
+	"\n"
 	"-e     --ecb            Cross-compile from an ASCII platform to\n"
 	"                        an EBCDIC one.\n"
 	"\n"
 	"-f     --storable-state Generate a scanner that supports storable states.\n"
 	"\n"
+	"-F     --flex-syntax    Partial support for flex syntax.\n"
+	"\n"
 	"-g     --computed-gotos Implies -b. Generate computed goto code (only useable\n"
 	"                        with gcc).\n"
 	"\n"
@@ -143,6 +164,8 @@ static void usage()
 	"-o     --output=output  Specify the output file instead of stdout\n"
 	"                        This cannot be used together with -e switch.\n"
 	"\n"
+	"-r     --reusable       Allow reuse of scanner definitions.\n"
+	"\n"
 	"-s     --nested-ifs     Generate nested ifs for some switches. Many compilers\n"
 	"                        need this assist to generate better code.\n"
 	"\n"
@@ -163,6 +186,13 @@ static void usage()
 	"\n"
 	"--no-generation-date    Suppress date output in the generated output so that it\n"
 	"                        only shows the re2c version.\n"
+	"\n"
+	"--case-insensitive      All strings are case insensitive, so all \"-expressions\n"
+	"                        are treated in the same way '-expressions are.\n"
+	"\n"
+	"--case-inverted         Invert the meaning of single and double quoted strings.\n"
+	"                        With this switch single quotes are case sensitive and\n"
+	"                        double quotes are case insensitive.\n"
 	;
 }
 
@@ -207,10 +237,19 @@ int main(int argc, char *argv[])
 			dFlag = true;
 			break;
 
+			case 'D':
+			DFlag = true;
+			iFlag = true;
+			break;
+
 			case 'f':
 			fFlag = true;
 			break;
 
+			case 'F':
+			FFlag = true;
+			break;
+
 			case 'g':
 			gFlag = true;
 			bFlag = true;
@@ -225,6 +264,10 @@ int main(int argc, char *argv[])
 			outputFileName = opt_arg;
 			break;
 
+			case 'r':
+			rFlag = true;
+			break;
+
 			case 's':
 			sFlag = true;
 			break;
@@ -285,6 +328,14 @@ int main(int argc, char *argv[])
 			case 10:
 			bNoGenerationDate = true;
 			break;
+
+			case 11:
+			bCaseInsensitive = true;
+			break;
+
+			case 12:
+			bCaseInverted = true;
+			break;
 		}
 	}
 
@@ -309,6 +360,12 @@ int main(int argc, char *argv[])
 		return 2;
 	}
 
+	if (dFlag && DFlag)
+	{
+		std::cerr << "re2c: error: Cannot combine -d with -D switch\n";
+		return 2;
+	}
+
 	if (uFlag)
 	{
 		wFlag = true;
