@@ -1,11 +1,10 @@
-/* $Id: parser.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: parser.h 690 2007-04-22 14:56:40Z helly $ */
 #ifndef _parser_h
 #define _parser_h
 
 #include "scanner.h"
 #include "re.h"
 #include <iosfwd>
-#include <map>
 
 namespace re2c
 {
@@ -20,6 +19,11 @@ public:
 	static void ClearTable();
 
 	typedef std::map<std::string, Symbol*> SymbolTable;
+	
+	const Str& GetName() const
+	{
+		return name;
+	}
 
 protected:
 
@@ -49,7 +53,7 @@ private:
 #endif
 };
 
-void parse(Scanner&, std::ostream&);
+extern void parse(Scanner&, std::ostream&, std::ostream*);
 
 } // end namespace re2c
 
